/**
 * 
 */
package com.example.local.service;

import javax.xml.ws.WebServiceRef;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.example.local.dao.ILocalDAO;
import com.example.local.exception.LocalException;
import com.example.local.model.LocalAccount;
import com.example.server.webservice.RemoteAccountWebService;
import com.example.server.webservice.RemoteAccountWebService_Service;

/**
 * @author PradeepBadiger
 * 
 */
@Service
public class LocalService {

	private static final Logger LOGGER = Logger.getLogger(LocalService.class);

	@WebServiceRef(value = RemoteAccountWebService_Service.class)
	private RemoteAccountWebService serverWebRef;

	@Autowired
	private ILocalDAO localDAO;

	/**
	 * Saves and return local account.
	 * 
	 * @param localAccount
	 * @param testCaseId
	 * @return
	 * @throws LocalException
	 */
	@Transactional(propagation = Propagation.REQUIRES_NEW, rollbackFor = LocalException.class)
	public LocalAccount saveLocalAccount(LocalAccount localAccount,
			String testCaseId) throws LocalException {
		LOGGER.info("Calling Remote Account web service....");
		try {
			String status  = serverWebRef.saveAccountRecord(
					localAccount.getAccountNumber(),
					localAccount.getAmount(),
					"TxId" + localAccount.getAccountNumber(),
					testCaseId);
			LOGGER.info("Remote account web service returned : " + status);
		} catch (Exception e) {
			LOGGER.error(e);
			throw new LocalException(e);
		}

		LOGGER.info("Saving Local Account object......\n");
		LocalAccount account = localDAO.saveLocalRecord(localAccount);

		if ("LOCAL_ROLLBACK".equals(testCaseId)) {
			throw new LocalException("Local rollback data.");
		}

		LOGGER.info("Saved Local Account object successfully.");
		return account;
	}

}
